/* 
   KOPPELO-tietokannan luomiseen kytettvt SQL-lauseet
   Kirjoitettu 14.3.2002
   Pivitetty 2.5.2002

   Taulujen nimemisest
   =====================

   Taulut on nimetty kytttarkoituksen mukaan. Useisiin taulujen nimiin liittyy 
   etuliite Assignment tai Ass, jotta KOPPELO-opinnytekokonaisuuden muodostavat 
   taulut olisi helposti erotettavissa muista KOTKA-tietokannan tauluista. 
   Etuliitett Assignment kytetn silloin, kun taulun nimen loppuosa muodostuu 
   yhdest lyhyest sanasta. Lyhennett Ass kytetn useampiosaisten tai muutoin 
   pitkien nimien yhteydess.
 
   Jlkiliite Translation taulun perss ilmaisee, ett kyseisess taulussa 
   silytetn ko. tiedon knnksi.

   Viiteavainten nimemisest
   ==========================

   Taulujen ensisijaiset avaimet (primary key) on nimetty lismll taulun nimeen 
   kirjaimet PK.   

   Toissijaiset avaimet (foreign key) on nimetty kyttmll taulun nimen jljess 
   kirjaimia FK ja kyseisen alkion kolmea ensimmist kirjainta. Niiss harvoissa
   tapauksissa, joissa taulussa on kaksi toissijaista viiteavainta joiden nimi alkaa 
   samoilla merkeill, on snt hieman sovellettu.

   Ryhmittelyst
   =============
   
   KOPPELO-kannan taulut on jaettu viiteen ryhmn sen perusteella, kuinka taulujen 
   luontijrjestys vaikuttaa muihin tauluihin. Ryhmn n taulut voidaan luoda periaat-
   teessa miss tahansa jrjestyksess muihin n-ryhmn tauluihin nhden (poikkeuksena 
   ryhmn II AssTypeTranslation-taulu).

   Viimeisenn ryhmn on KOPPELO-sovellukseen liittyv oikeustaulujen ryhm, jossa
   taulujen luontijrjestys merkitsee. 

*/


/* Ryhm I */

CREATE TABLE AssignmentType (
Deleted           BOOLEAN NOT NULL,
TypeId            INTEGER NOT NULL,
DefaultTimeInDB   INTEGER NOT NULL, /* aika, jonka kuluttua aihe poistuu (vrk) */
MaxTimeInDB       INTEGER NOT NULL, /* maksimiaika, joka aiheelle voidaan asettaa */ 
CONSTRAINT AssignmentTypePK
  PRIMARY KEY (TypeID)
);

CREATE TABLE AssignmentOriginal (
Deleted           BOOLEAN NOT NULL,
OriginalID        INTEGER NOT NULL,
ProposedDate      DATE NOT NULL,
ProposedBy        INTEGER NULL,  /* vai pitk olla NOT NULL? */
AcceptedDate      DATE NULL,     /* pit saada olla NULL ehdotetulle aiheelle */
AcceptedBy        INTEGER NULL,  /* -||- */
ValidUntil        DATE NOT NULL,
Available         INTEGER NOT NULL,
CONSTRAINT AssignmentOriginalPK
  PRIMARY KEY (OriginalID),
CONSTRAINT AssignmentOriginalFKPro
  FOREIGN KEY (ProposedBy)
  REFERENCES Person(PersonID)
  ON DELETE NO ACTION
  ON UPDATE CASCADE,
CONSTRAINT AssignmentOriginalFKAcc
  FOREIGN KEY (AcceptedBy)
  REFERENCES Person(PersonID)
  ON DELETE NO ACTION
  ON UPDATE CASCADE
);

CREATE TABLE StudyLine (
Deleted           BOOLEAN NOT NULL,
LineID            INTEGER NOT NULL,
OrganisationID    INTEGER NOT NULL,
CONSTRAINT StudyLinePK
  PRIMARY KEY (LineID),
CONSTRAINT StudyLineFKOrg
  FOREIGN KEY (OrganisationID)
  REFERENCES Organisation(OrganisationID) 
  ON DELETE NO ACTION
  ON UPDATE CASCADE
);

CREATE TABLE AssignmentPhase (
Deleted           BOOLEAN NOT NULL,
PhaseID           INTEGER NOT NULL,
PhaseGroup        INTEGER NOT NULL,
CONSTRAINT AssignmentPhasePK
  PRIMARY KEY (PhaseID)
);

CREATE TABLE AssTaskCategory (
Deleted           BOOLEAN NOT NULL,
CategoryID        INTEGER NOT NULL,
CONSTRAINT AssTaskCategoryPK
  PRIMARY KEY (CategoryID)
);

CREATE TABLE AssTaskTemplate (
Deleted           BOOLEAN NOT NULL,
TemplateID        INTEGER NOT NULL,
CONSTRAINT AssignmentModelPK
  PRIMARY KEY (TemplateID)
);

CREATE TABLE AssignmentRole (
Deleted           BOOLEAN NOT NULL,
RoleID            INTEGER NOT NULL,
CONSTRAINT AssignmentRolePK
  PRIMARY KEY (RoleID)
);

CREATE TABLE AssEventType (
Deleted           BOOLEAN NOT NULL,
TypeID            INTEGER NOT NULL,
CONSTRAINT AssEventTypePK
  PRIMARY KEY (TypeID)
);

CREATE TABLE AssignmentKeyword (
Deleted           BOOLEAN NOT NULL,
KeywordID         INTEGER NOT NULL,
CONSTRAINT AssignmentKeywordPK
  PRIMARY KEY (KeywordID)
);

CREATE TABLE AssignmentDetail (
Deleted           BOOLEAN NOT NULL,
DetailID          INTEGER NOT NULL,
TypeID            INTEGER NOT NULL,
CONSTRAINT AssignmentDetailPK
  PRIMARY KEY (DetailID),
CONSTRAINT AssignmentDetailFKTyp
  FOREIGN KEY (TypeID)
  REFERENCES DescriptionType(DescriptionTypeID) 
  ON DELETE NO ACTION
  ON UPDATE CASCADE
);

/* Ryhm II */ 

CREATE TABLE Assignment (
Deleted           BOOLEAN NOT NULL,
AssignmentID      INTEGER NOT NULL,
WarningTime       INTEGER NULL,     /* maksimiaika kahden tapahtuman vlill (vrk) */
PublicGroup       INTEGER NOT NULL, /* nkyvyys-tagi */
OriginalID        INTEGER NOT NULL, /* alkuperinen aihe */ 
AssignmentTypeID  INTEGER NOT NULL, /* opinnytteen tyyppi, tasan yksi */  
CourseID          INTEGER NOT NULL, /* tarvitaan kurssikoodiin */   
LineID            INTEGER NULL,     /* kaikkiin ei tarvitse kiinnitt linjaa */
/*PeriodID          INTEGER NULL,      viittaa KOTKA-kannan ajanjaksoon, ei pakollinen */ 
BeginDate         DATE    NOT NULL, /* seuraavilla korvataan KOTKA-kannan period-ksite */
FinishedDate      DATE    NULL, 
PhaseID           INTEGER NOT NULL,
CONSTRAINT AssignmentPK
  PRIMARY KEY (AssignmentID),
CONSTRAINT AssignmentFKStu
  FOREIGN KEY (OriginalID)
  REFERENCES AssignmentOriginal(OriginalID)
  ON DELETE NO ACTION
  ON UPDATE CASCADE,
CONSTRAINT AssigmentFKTyp
  FOREIGN KEY (AssignmentTypeID)
  REFERENCES AssignmentType(TypeID)
  ON DELETE NO ACTION
  ON UPDATE CASCADE,
CONSTRAINT AssigmentFKCou
  FOREIGN KEY (CourseID)
  REFERENCES Course(CourseID)
  ON DELETE NO ACTION
  ON UPDATE CASCADE,
CONSTRAINT AssigmentFKLin
  FOREIGN KEY (LineID)
  REFERENCES StudyLine(LineID)
  ON DELETE NO ACTION
  ON UPDATE CASCADE,
/* CONSTRAINT AssigmentFKPer      
  FOREIGN KEY (PeriodID)
  REFERENCES Period(PeriodID)
  ON DELETE NO ACTION
  ON UPDATE CASCADE, */	
CONSTRAINT AssigmentFKPha
  FOREIGN KEY (PhaseID)
  REFERENCES AssignmentPhase(PhaseID)
  ON DELETE NO ACTION
  ON UPDATE CASCADE
);

CREATE TABLE AssignmentTask (
Deleted           BOOLEAN NOT NULL,
TaskID            INTEGER NOT NULL,
CategoryID        INTEGER NOT NULL,
CONSTRAINT AssignmentTaskPK
  PRIMARY KEY (TaskID),
CONSTRAINT AssignmentTaskFKCat
  FOREIGN KEY (CategoryID)
  REFERENCES AssTaskCategory(CategoryID) 
  ON DELETE NO ACTION
  ON UPDATE CASCADE
);

CREATE TABLE AssOriginalPosLine (
Deleted           BOOLEAN NOT NULL,
OriginalID        INTEGER NOT NULL,
LineID            INTEGER NOT NULL,
CONSTRAINT AssOriginalPosLinePK
  PRIMARY KEY (OriginalID, LineID),
CONSTRAINT AssOriginalPosLineFKOri
  FOREIGN KEY (OriginalID)
  REFERENCES AssignmentOriginal(OriginalID) 
  ON DELETE NO ACTION
  ON UPDATE CASCADE,
CONSTRAINT AssOriginalPosLineFKLin
  FOREIGN KEY (LineID)
  REFERENCES StudyLine(LineID) 
  ON DELETE NO ACTION
  ON UPDATE CASCADE
);

CREATE TABLE AssOriginalPosType (
Deleted           BOOLEAN NOT NULL,
OriginalID        INTEGER NOT NULL,
TypeID            INTEGER NOT NULL,
CONSTRAINT AssOriginalPosTypePK
  PRIMARY KEY (OriginalID, TypeID),
CONSTRAINT AssOriginalPosTypeFKOri
  FOREIGN KEY (OriginalID)
  REFERENCES AssignmentOriginal(OriginalID) 
  ON DELETE NO ACTION
  ON UPDATE CASCADE,
CONSTRAINT AssOriginalPosTypeFKTyp
  FOREIGN KEY (TypeID)
  REFERENCES AssignmentType(TypeID) 
  ON DELETE NO ACTION
  ON UPDATE CASCADE
);

CREATE TABLE AssTypeTranslation (
Deleted           BOOLEAN NOT NULL,
TypeID            INTEGER NOT NULL,
LanguageID        INTEGER NOT NULL,
Name              VARCHAR NOT NULL,  
CONSTRAINT AssTypeTranslationPK
  PRIMARY KEY (TypeID,LanguageID),
CONSTRAINT AssTypeTranslationFKTyp
  FOREIGN KEY (TypeID)
  REFERENCES AssignmentType(TypeID)
  ON DELETE NO ACTION
  ON UPDATE CASCADE,
CONSTRAINT AssTypeTranslationFKLan
  FOREIGN KEY (LanguageID)
  REFERENCES Language(LanguageID)
  ON DELETE NO ACTION
  ON UPDATE CASCADE
);

CREATE TABLE StudyLineTranslation (
Deleted           BOOLEAN NOT NULL,
LineID            INTEGER NOT NULL,
LanguageID        INTEGER NOT NULL,
Name              VARCHAR NOT NULL,  
CONSTRAINT StudyLineTranslationPK
  PRIMARY KEY (LineID,LanguageID),
CONSTRAINT StudyLineTranslationFKLin
  FOREIGN KEY (LineID)
  REFERENCES StudyLine(LineID)
  ON DELETE NO ACTION
  ON UPDATE CASCADE,	
CONSTRAINT StudyLineTranslationFKLan
  FOREIGN KEY (LanguageID)
  REFERENCES Language(LanguageID)
  ON DELETE NO ACTION
  ON UPDATE CASCADE
);

CREATE TABLE AssPhaseTranslation (
Deleted           BOOLEAN NOT NULL,
PhaseID           INTEGER NOT NULL,
LanguageID        INTEGER NOT NULL,
Name              VARCHAR NOT NULL,  
CONSTRAINT PhaseTranslationPK
  PRIMARY KEY (PhaseID,LanguageID),
CONSTRAINT PhaseTranslationFKPha
  FOREIGN KEY (PhaseID)
  REFERENCES AssignmentPhase(PhaseID)
  ON DELETE NO ACTION
  ON UPDATE CASCADE,	
CONSTRAINT PhaseTranslationFKLan
  FOREIGN KEY (LanguageID)
  REFERENCES Language(LanguageID)
  ON DELETE NO ACTION
  ON UPDATE CASCADE
);

CREATE TABLE AssTaskTemplateTranslation (
Deleted           BOOLEAN NOT NULL,
TemplateID        INTEGER NOT NULL,
LanguageID        INTEGER NOT NULL,
Name              VARCHAR NOT NULL,  
CONSTRAINT AssTaskTemplateTranslationPK
  PRIMARY KEY (TemplateID,LanguageID),
CONSTRAINT AssTaskTemplateTranslationFKTem
  FOREIGN KEY (TemplateID)
  REFERENCES AssTaskTemplate(TemplateID)
  ON DELETE NO ACTION
  ON UPDATE CASCADE,
CONSTRAINT AssTaskTemplateTranslationFKLan
  FOREIGN KEY (LanguageID)
  REFERENCES Language(LanguageID)
  ON DELETE NO ACTION
  ON UPDATE CASCADE	
);

CREATE TABLE AssTaskTranslation (
Deleted           BOOLEAN NOT NULL,
TaskID            INTEGER NOT NULL,
LanguageID        INTEGER NOT NULL,
Name              VARCHAR NOT NULL,  
CONSTRAINT AssTaskTranslationPK
  PRIMARY KEY (TaskID,LanguageID),
CONSTRAINT AssTaskTranslationFKTas
  FOREIGN KEY (TaskID)
  REFERENCES AssignmentTask(TaskID)
  ON DELETE NO ACTION
  ON UPDATE CASCADE,
CONSTRAINT AssTaskTranslationFKLan
  FOREIGN KEY (LanguageID)
  REFERENCES Language(LanguageID)
  ON DELETE NO ACTION
  ON UPDATE CASCADE	
);

CREATE TABLE AssTaskCategoryTranslation (
Deleted           BOOLEAN NOT NULL,
CategoryID        INTEGER NOT NULL,
LanguageID        INTEGER NOT NULL,
Name              VARCHAR NOT NULL,  
CONSTRAINT AssTaskCategoryTranslationPK
  PRIMARY KEY (CategoryID,LanguageID),
CONSTRAINT AssTaskCategoryTranslationFKCat
  FOREIGN KEY (CategoryID)
  REFERENCES AssTaskCategory(CategoryID)
  ON DELETE NO ACTION
  ON UPDATE CASCADE,
CONSTRAINT AssTaskCategoryTranslationFKLan
  FOREIGN KEY (LanguageID)
  REFERENCES Language(LanguageID)
  ON DELETE NO ACTION
  ON UPDATE CASCADE	
);

CREATE TABLE AssEventTypeTranslation (
Deleted           BOOLEAN NOT NULL,
TypeID            INTEGER NOT NULL,
LanguageID        INTEGER NOT NULL,
Name              VARCHAR NOT NULL,  
CONSTRAINT AssEventTypeTranslationPK
  PRIMARY KEY (TypeID,LanguageID),
CONSTRAINT AssEventTypeTranslationTypeFKTyp
  FOREIGN KEY (TypeID)
  REFERENCES AssEventType(TypeID)
  ON DELETE NO ACTION
  ON UPDATE CASCADE,
CONSTRAINT AssEventTypeTranslationFKLan
  FOREIGN KEY (LanguageID)
  REFERENCES Language(LanguageID)
  ON DELETE NO ACTION
  ON UPDATE CASCADE	
);

CREATE TABLE AssRoleTranslation (
Deleted           BOOLEAN NOT NULL,
RoleID            INTEGER NOT NULL,
LanguageID        INTEGER NOT NULL,
Name              VARCHAR NOT NULL,  
CONSTRAINT AssignmentRoleTranslationPK
  PRIMARY KEY (RoleID,LanguageID),
CONSTRAINT AssignmentRoleTranslationFKRol
  FOREIGN KEY (RoleID)
  REFERENCES AssignmentRole(RoleID)
  ON DELETE NO ACTION
  ON UPDATE CASCADE,
CONSTRAINT AssigmentRoleTranslationFKLan
  FOREIGN KEY (LanguageID)
  REFERENCES Language(LanguageID)
  ON DELETE NO ACTION
  ON UPDATE CASCADE	
);

CREATE TABLE AssKeywordTranslation (
Deleted           BOOLEAN NOT NULL,
KeywordID         INTEGER NOT NULL,
LanguageID        INTEGER NOT NULL,
Name              VARCHAR NOT NULL,  
CONSTRAINT AssKeywordTranslationPK
  PRIMARY KEY (KeywordID,LanguageID),
CONSTRAINT AssignmentKeywordTranslationFKKey
  FOREIGN KEY (KeywordID)
  REFERENCES AssignmentKeyword(KeywordID)
  ON DELETE NO ACTION
  ON UPDATE CASCADE,
CONSTRAINT AssKeywordTranslationFKLan
  FOREIGN KEY (LanguageID)
  REFERENCES Language(LanguageID)
  ON DELETE NO ACTION
  ON UPDATE CASCADE	
);

CREATE TABLE AssDetailTranslation (
Deleted           BOOLEAN NOT NULL,
DetailID          INTEGER NOT NULL,
LanguageID        INTEGER NOT NULL,
Name              VARCHAR NOT NULL,  
CONSTRAINT AssDetailTranslationPK
  PRIMARY KEY (DetailID,LanguageID),
CONSTRAINT AssignmentDetailTranslationFKKey
  FOREIGN KEY (DetailID)
  REFERENCES AssignmentDetail(DetailID)
  ON DELETE NO ACTION
  ON UPDATE CASCADE,
CONSTRAINT AssKeywordTranslationFKLan
  FOREIGN KEY (LanguageID)
  REFERENCES Language(LanguageID)
  ON DELETE NO ACTION
  ON UPDATE CASCADE	
);


/* Ryhm III */

CREATE TABLE AssOriginalRelKeyword (
Deleted           BOOLEAN NOT NULL,
OriginalID        INTEGER NOT NULL,
KeywordID         INTEGER NOT NULL,
CONSTRAINT AssOriginalRelKeywordPK
  PRIMARY KEY (OriginalID, KeywordID),
CONSTRAINT AssOriginalRelKeywordFKKey
  FOREIGN KEY (KeywordID)
  REFERENCES AssignmentKeyword(KeywordID)
  ON DELETE NO ACTION
  ON UPDATE CASCADE,
CONSTRAINT AssOriginalRelKeywordFKOri
  FOREIGN KEY (OriginalID)
  REFERENCES AssignmentOriginal(OriginalID)
  ON DELETE NO ACTION
  ON UPDATE CASCADE
);

CREATE TABLE AssignmentRelKeyword (
Deleted           BOOLEAN NOT NULL,
AssignmentID      INTEGER NOT NULL,
KeywordID         INTEGER NOT NULL,
CONSTRAINT AssignmentRelKeywordPK
  PRIMARY KEY (AssignmentID, KeywordID),
CONSTRAINT AssignmentRelKeywordFKKey
  FOREIGN KEY (KeywordID)
  REFERENCES AssignmentKeyword(KeywordID)
  ON DELETE NO ACTION
  ON UPDATE CASCADE,
CONSTRAINT AssignmenRelKeywordFKOri
  FOREIGN KEY (AssignmentID)
  REFERENCES Assignment(AssignmentID)
  ON DELETE NO ACTION
  ON UPDATE CASCADE
);

CREATE TABLE AssignmentRelDetail (
Deleted           BOOLEAN NOT NULL,
AssignmentID      INTEGER NOT NULL,
DetailID          INTEGER NOT NULL,
AllowUpdate       BOOLEAN NOT NULL,
CONSTRAINT AssignmentRelDetailPK
  PRIMARY KEY (AssignmentID, DetailID),
CONSTRAINT AssignmentRelDetailFKAss
  FOREIGN KEY (AssignmentID)
  REFERENCES Assignment(AssignmentID)
  ON DELETE NO ACTION
  ON UPDATE CASCADE,
CONSTRAINT AssignmentRelDetailFKDet
  FOREIGN KEY (DetailID)
  REFERENCES AssignmentDetail(DetailID)
  ON DELETE NO ACTION
  ON UPDATE CASCADE
);

CREATE TABLE AssOriginalRelDetail (
Deleted           BOOLEAN NOT NULL,
OriginalID        INTEGER NOT NULL,
DetailID          INTEGER NOT NULL,
CONSTRAINT AssOriginalRelDetailPK
  PRIMARY KEY (OriginalID, DetailID),
CONSTRAINT AssOriginalRelDetailFKOri
  FOREIGN KEY (OriginalID)
  REFERENCES AssignmentOriginal(OriginalID)
  ON DELETE NO ACTION
  ON UPDATE CASCADE,
CONSTRAINT AssOriginalRelDetailFKDet
  FOREIGN KEY (DetailID)
  REFERENCES AssignmentDetail(DetailID)
  ON DELETE NO ACTION
  ON UPDATE CASCADE
);

CREATE TABLE AssTaskTemplateRelation (
Deleted           BOOLEAN NOT NULL,
TemplateID        INTEGER NOT NULL,
TaskID            INTEGER NOT NULL,
CONSTRAINT AssTaskTemplateRelationPK
  PRIMARY KEY (TemplateID, TaskID),
CONSTRAINT AssTaskTemplateRelationFKTem
  FOREIGN KEY (TemplateID)
  REFERENCES AssTaskTemplate(TemplateID)
  ON DELETE NO ACTION
  ON UPDATE CASCADE,
CONSTRAINT AssTaskTemplateRelationFKTas
  FOREIGN KEY (TaskID)
  REFERENCES AssignmentTask(TaskID)
  ON DELETE NO ACTION
  ON UPDATE CASCADE
);

CREATE TABLE AssignmentRelatedPerson (
Deleted           BOOLEAN NOT NULL,
RelationID        INTEGER NOT NULL,
AssignmentID      INTEGER NOT NULL,
PersonID          INTEGER NOT NULL,
RoleID            INTEGER NOT NULL,
CONSTRAINT AssignmentRelatedPersonPK
  PRIMARY KEY (RelationID),
CONSTRAINT AssignmentRelatedPersonFKAss
  FOREIGN KEY (AssignmentID)
  REFERENCES Assignment(AssignmentID)
  ON DELETE NO ACTION
  ON UPDATE CASCADE,
CONSTRAINT AssignmentRelatedPersonFKPer
  FOREIGN KEY (PersonID)
  REFERENCES Person(PersonID)
  ON DELETE NO ACTION
  ON UPDATE CASCADE,
CONSTRAINT AssignmentRelatedPersonFKRol
  FOREIGN KEY (RoleID)
  REFERENCES AssignmentRole(RoleID)
  ON DELETE NO ACTION
  ON UPDATE CASCADE,
CONSTRAINT AssignmentRelatedPersonConst
  UNIQUE (AssignmentID, RoleID, PersonID)
);

/* Ryhm IV */

CREATE TABLE AssignmentTaskRelation (
Deleted           BOOLEAN NOT NULL,
AssignmentID      INTEGER NOT NULL,
TaskID            INTEGER NOT NULL,
PhaseID           INTEGER NOT NULL,
CONSTRAINT AssignmentTaskRelationPK
  PRIMARY KEY (AssignmentID, TaskID),
CONSTRAINT AssignmentTaskRelationFKAss
  FOREIGN KEY (AssignmentID)
  REFERENCES Assignment(AssignmentID)
  ON DELETE NO ACTION
  ON UPDATE CASCADE,
CONSTRAINT AssignmentTaskRelationFKTas
  FOREIGN KEY (TaskID)
  REFERENCES AssignmentTask(TaskID)
  ON DELETE NO ACTION
  ON UPDATE CASCADE,
CONSTRAINT AssignmentTaskRelationFKPha
  FOREIGN KEY (PhaseID)
  REFERENCES AssignmentPhase(PhaseID)
  ON DELETE NO ACTION
  ON UPDATE CASCADE
);

CREATE TABLE AssignmentFinished (
Deleted            BOOLEAN NOT NULL,
PersonToBeGradedID INTEGER NOT NULL, /* arvosteltava henkil */
GradedByID         INTEGER NOT NULL, /* arvostelija */
GradeID            INTEGER NOT NULL, /* arvolause */
GradeDate          DATE    NOT NULL, /* arvostelupiv */
Credits            NUMERIC NOT NULL, /* oppilaan opintoviikot */ 
Memo               VARCHAR NULL,  
CONSTRAINT AssignmentGradePK
  PRIMARY KEY (PersonToBeGradedID, GradedByID),
CONSTRAINT AssignmentGradeFKRel
  FOREIGN KEY (PersonToBeGradedID)
  REFERENCES AssignmentRelatedPerson(RelationID)
  ON DELETE NO ACTION 
  ON UPDATE CASCADE,
CONSTRAINT AssignmentGradeFKGraBy
  FOREIGN KEY (GradedByID)
  REFERENCES AssignmentRelatedPerson(RelationID)
  ON DELETE NO ACTION
  ON UPDATE CASCADE,
CONSTRAINT AssignmentGradeFKGra
  FOREIGN KEY (GradeID)
  REFERENCES Grade(GradeID)
  ON DELETE NO ACTION 
  ON UPDATE CASCADE
);

CREATE TABLE AssignmentEvent (
Deleted           BOOLEAN NOT NULL,
EventID           INTEGER NOT NULL,
AssignmentID      INTEGER NOT NULL, /* liittyy aina opinnytteeseen */
RegistratorID     INTEGER NOT NULL, /* tapahtuman kirjaaja */
EventDate         DATE NULL,
BeginTime         TIME NULL,
EndTime           TIME NULL,
Title             VARCHAR NULL,
Memo              VARCHAR NULL,
TaskID            INTEGER NULL,     /* tapahtumaan liittyv tytehtv */
EventTypeID       INTEGER NOT NULL, /* tapahtuman tyyppi */
CONSTRAINT AssignmentEventPK
  PRIMARY KEY (EventID),
CONSTRAINT AssignmentEventFKAss
  FOREIGN KEY (AssignmentID)
  REFERENCES Assignment(AssignmentID)
  ON DELETE NO ACTION
  ON UPDATE CASCADE,
CONSTRAINT AssignmenteventFKReg
  FOREIGN KEY (RegistratorID)
  REFERENCES AssignmentRelatedPerson(RelationID)
  ON DELETE NO ACTION
  ON UPDATE CASCADE,
CONSTRAINT AssignmentEventFKTas
  FOREIGN KEY (TaskID)
  REFERENCES AssignmentTask(TaskID)
  ON DELETE NO ACTION
  ON UPDATE CASCADE,
CONSTRAINT AssignmentEventFKTyp
  FOREIGN KEY (EventTypeID)
  REFERENCES AssEventType(TypeID)
  ON DELETE NO ACTION
  ON UPDATE CASCADE
); 

/* Ryhm V */

CREATE TABLE UnreadMessages (
Deleted           BOOLEAN NOT NULL,
AssPersonID       INTEGER NOT NULL,  /* tietty ihminen opinnytteess */
EventID           INTEGER NOT NULL,
ToBeRemoved       DATE    NOT NULL,
CONSTRAINT UnreadMessagesPK
  PRIMARY KEY (AssPersonID, EventID),
CONSTRAINT UnreadMessagesFKRel
  FOREIGN KEY (AssPersonID)
  REFERENCES AssignmentRelatedPerson(RelationID)
  ON DELETE NO ACTION 
  ON UPDATE CASCADE,
CONSTRAINT UnreadMessagesFKEve
  FOREIGN KEY (EventID)
  REFERENCES AssignmentEvent(EventID)
  ON DELETE NO ACTION
  ON UPDATE CASCADE
);



/* Ryhm VI */

CREATE TABLE AssignmentRight (
Deleted           BOOLEAN NOT NULL,
RightID           INTEGER NOT NULL, /* oikeuden id */
Name              VARCHAR NOT NULL,
CONSTRAINT AssignmentRightPK
  PRIMARY KEY (RightID)
);

CREATE TABLE AssUserGroupRelRight (
Deleted           BOOLEAN NOT NULL,
GroupID           INTEGER NOT NULL,  /* KOTKAn kyttjryhm */
RightID           INTEGER NOT NULL,  /* oikeus */
GrantedBy         INTEGER NOT NULL,  /* oikeuden antaja */ 
GrantedOn         DATE    NOT NULL,
CONSTRAINT AssUserGroupRelRightPK
  PRIMARY KEY (GroupID, RightID),
CONSTRAINT AssUserGroupRelRighFKRig
  FOREIGN KEY (RightID)
  REFERENCES AssignmentRight(RightID)
  ON DELETE NO ACTION 
  ON UPDATE CASCADE,
CONSTRAINT AssUserGroupRelRighFKGro
  FOREIGN KEY (GroupID)
  REFERENCES AccessRightLevel(AccessRightLevelID)
  ON DELETE NO ACTION 
  ON UPDATE CASCADE,
CONSTRAINT AssUserGroupRelRighFKGra
  FOREIGN KEY (GrantedBy)
  REFERENCES Person(PersonID)
  ON DELETE NO ACTION
  ON UPDATE CASCADE
);

CREATE TABLE AssRoleRelRight (
Deleted           BOOLEAN NOT NULL,
RoleID            INTEGER NOT NULL,  /* opinnytteeseen littyv kyttjryhm */
RightID           INTEGER NOT NULL,  /* oikeus */
GrantedBy         INTEGER NOT NULL,  /* oikeuden antaja */ 
GrantedOn         DATE    NOT NULL,
CONSTRAINT AssRoleRelRightPK
  PRIMARY KEY (RoleID, RightID),
CONSTRAINT AssRoleRelRighFKRig
  FOREIGN KEY (RightID)
  REFERENCES AssignmentRight(RightID)
  ON DELETE NO ACTION 
  ON UPDATE CASCADE,
CONSTRAINT AssRoleRelRighFKRol
  FOREIGN KEY (RoleID)
  REFERENCES AssignmentRole(RoleID)
  ON DELETE NO ACTION 
  ON UPDATE CASCADE,
CONSTRAINT AssRoleRelRighFKGra
  FOREIGN KEY (GrantedBy)
  REFERENCES Person(PersonID)
  ON DELETE NO ACTION
  ON UPDATE CASCADE
);

CREATE TABLE AssPersonRelRight (
Deleted           BOOLEAN NOT NULL,
PersonID          INTEGER NOT NULL,  /* yksittinen henkil */
RightID           INTEGER NOT NULL,  /* oikeus */
LineID            INTEGER NOT NULL,  /* mill linjalla */
AssTypeID         INTEGER NOT NULL,  /* mill opinnytetyypill */
GrantedBy         INTEGER NOT NULL,  /* oikeuden antaja */ 
GrantedOn         DATE    NOT NULL,
CONSTRAINT AssPersonRelRightPK
  PRIMARY KEY (PersonID, RightID, LineID, AssTypeID),
CONSTRAINT AssPersonRelRighFKRig
  FOREIGN KEY (RightID)
  REFERENCES AssignmentRight(RightID)
  ON DELETE NO ACTION 
  ON UPDATE CASCADE,
CONSTRAINT AssPersonRelRighFKLin
  FOREIGN KEY (LineID)
  REFERENCES StudyLine(LineID)
  ON DELETE NO ACTION 
  ON UPDATE CASCADE,
CONSTRAINT AssPersonRelRighFKTyp
  FOREIGN KEY (AssTypeID)
  REFERENCES AssignmentType(TypeID)
  ON DELETE NO ACTION 
  ON UPDATE CASCADE,
CONSTRAINT AssPersonRelRighFKGra
  FOREIGN KEY (GrantedBy)
  REFERENCES Person(PersonID)
  ON DELETE NO ACTION
  ON UPDATE CASCADE
);




